'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CancelToken = _axios2.default.CancelToken;

var EndPoint = function () {
    function EndPoint(name, _ref) {
        var _this = this;

        var url = _ref.url;
        var requestKey = _ref.requestKey;
        var defaultRequestConfig = _ref.defaultRequestConfig;

        _classCallCheck(this, EndPoint);

        this.name = name;
        this.url = url;
        this.defaultRequestConfig = defaultRequestConfig || {};
        this.isMultiRequest = !!requestKey;
        this.requestKey = requestKey || function () {
            return 'default';
        };
        this.cancellation = {};

        // Construct the methods for http methods that don't need data
        ['delete', 'get', 'head', 'options'].forEach(function (method) {
            _this[method] = function (args) {
                var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
                return function (dispatch) {
                    return _this.request(dispatch, method, args, config);
                };
            };
        });

        // Construct the methods for http methods that do need data
        ['post', 'put', 'patch'].forEach(function (method) {
            _this[method] = function (args, data) {
                var config = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
                return function (dispatch) {
                    return _this.request(dispatch, method, args, config, data);
                };
            };
        });
    }

    _createClass(EndPoint, [{
        key: 'cancel',
        value: function cancel(args) {
            var cancellation = this.cancellation[this.requestKey(args)];
            cancellation && cancellation.cancel();
        }
    }, {
        key: 'createCancellation',
        value: function createCancellation(args) {
            var cancellation = CancelToken.source();
            this.cancellation[this.requestKey(args)] = cancellation;
            return cancellation;
        }
    }, {
        key: 'transformUrl',
        value: function transformUrl() {
            var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

            var url = this.url;
            // Replace all :arg with the actual value
            Object.keys(args).forEach(function (key) {
                var argRe = new RegExp(':' + key + '(?=[^\w]|$)');
                url = url.replace(argRe, args[key]);
            });
            // Remove remaining arguments not present in args
            url = url.replace(/:[\w]+(?=[^\w]|$)/, '');
            // Remove double slashes
            url = url.replace(/\/+/g, '/');
            return url;
        }
    }, {
        key: 'actionPrefix',
        value: function actionPrefix() {
            return '@@super-api@' + this.name + '_';
        }
    }, {
        key: 'isValidActionType',
        value: function isValidActionType(actionType) {
            return actionType && actionType.startsWith(this.actionPrefix());
        }
    }, {
        key: 'actionType',
        value: function actionType(action) {
            return this.actionPrefix() + action;
        }
    }, {
        key: 'actionSuccess',
        value: function actionSuccess(response, args) {
            return {
                type: this.actionType('success'),
                data: response.data,
                status: response.status,
                args: args
            };
        }
    }, {
        key: 'actionError',
        value: function actionError(error, args) {
            if (error.response) {
                // Server responded with an error
                return {
                    type: this.actionType('error'),
                    error: error.response.data,
                    status: error.response.status,
                    args: args
                };
            } else {
                // Request was malformed
                return {
                    type: this.actionType('error'),
                    error: error.message,
                    args: args
                };
            }
        }
    }, {
        key: 'actionRequest',
        value: function actionRequest(method, args) {
            return {
                type: this.actionType('request'),
                method: method,
                args: args
            };
        }
    }, {
        key: 'actionReset',
        value: function actionReset(args) {
            return {
                type: this.actionType('reset'),
                args: args
            };
        }
    }, {
        key: 'reset',
        value: function reset(args) {
            var _this2 = this;

            return function (dispatch) {
                _this2.cancel(args);
                dispatch(_this2.actionReset(args));
            };
        }
    }, {
        key: 'getOnce',
        value: function getOnce(args) {
            var _this3 = this;

            var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

            // Will fetch data only if it hasn't been fetched and hasn't started fetching.
            return function (dispatch, getState) {
                return _this3.requestOnce(dispatch, getState, 'get', args, config);
            };
        }
    }, {
        key: 'requestOnce',
        value: function requestOnce(dispatch, getState, method, args, config) {
            var data = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : undefined;

            // Start the request only if there is no data already loaded or loading
            var key = this.requestKey(args);
            if (this.cancellation[key]) {
                // A request is already ongoing
                return Promise.resolve();
            }
            var state = getState();
            if (this.isMultiRequest && state[key] && state[key].sync) {
                // There is no pending request but data has already been loaded
                return Promise.resolve();
            }
            if (!this.isMultiRequest && state.sync) {
                // There is no pending request but data has already been loaded
                return Promise.resolve();
            }

            return this.request(dispatch, method, args, config, data);
        }
    }, {
        key: 'request',
        value: function request(dispatch, method, args, config) {
            var _this4 = this;

            var data = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : undefined;

            // Cancel any pending request
            this.cancel(args);

            dispatch(this.actionRequest(method, args));

            return _axios2.default.request(_extends({
                url: this.transformUrl(args),
                method: method,
                data: data,
                cancelToken: this.createCancellation(args).token
            }, this.defaultRequestConfig[method], config)).catch(function (error) {
                // First handle "operational errors", where the request did not complete as expected.
                dispatch(_this4.actionError(error, args));
                throw error; // rethrow error for chaining
            }).then(function (response) {
                // The .then handler happens after handling errors from the server ("operational errors") so that we
                // don't accidentally catch "programmers error".
                // See http://www.2ality.com/2016/03/promise-rejections-vs-exceptions.html
                dispatch(_this4.actionSuccess(response, args));
            });
        }
    }, {
        key: 'reduce',
        value: function reduce(state, action) {
            if (this.isMultiRequest) {
                return this.reduceMultiRequest(state, action);
            } else {
                return this.reduceRequest(state, action);
            }
        }
    }, {
        key: 'reduceMultiRequest',
        value: function reduceMultiRequest(state, action) {
            if (typeof state === 'undefined') {
                state = {};
            }

            if (!this.isValidActionType(action.type)) {
                // Don't try to reduce an action coming from a different endpoint.
                return state;
            } else {
                var key = this.requestKey(action.args);

                return _extends({}, state, _defineProperty({}, key, this.reduceRequest(state[key], action)));
            }
        }
    }, {
        key: 'reduceRequest',
        value: function reduceRequest(state, action) {
            var defaultState = {
                loading: false,
                sync: false,
                syncing: false,
                data: {},
                error: null
            };

            if (typeof state === 'undefined') {
                state = defaultState;
            }

            switch (action.type) {
                case this.actionType('request'):
                    return _extends({}, state, {
                        loading: true,
                        sync: false,
                        syncing: true,
                        error: null
                    });
                case this.actionType('success'):
                    return _extends({}, state, {
                        loading: false,
                        sync: true,
                        syncing: false,
                        data: action.data,
                        error: null
                    });
                case this.actionType('error'):
                    return _extends({}, state, {
                        loading: false,
                        sync: false,
                        syncing: false,
                        error: action.error
                    });
                case this.actionType('reset'):
                    return defaultState;
                default:
                    return state;
            }
        }
    }]);

    return EndPoint;
}();

EndPoint.axios = _axios2.default; // used for mocking in unit-tests


exports.default = EndPoint;